#!/usr/bin/perl

#
# Copyright (C) 2000 Serge N. Pokhodyaev
#
# Distributed under GNU GPL
#
# E-mail: snp@ru.ru
# FIDO:   2:5020/1838
#
# $Id: report_traffic.pl,v 1.2 2000/08/19 04:33:03 snp Exp $
#

require 5.000;

my $VERSION = '$Revision: 1.2 $';
my $PROGRAM = "report_traffic";

use POSIX qw(strftime);

# Configurable part ##########################################################

my $msgfrom = "\"Stat daemon\" <root\@gloom.intra.eu.org>";
my $msgnewsgroup = "ftn.1838.stat";
my $msgnode = "2:5020/1838.0";

##############################################################################

# Common configuration for perl scripts
#<INCLUDE config.pl>

#my $logdir = CONFIG_get("LOGDIR");
my $logdir = "/var/log/fidogate";
my $log;
my $logtime = time - 86400;
my $logdate1 = strftime("%b %_e", localtime($logtime));
my $logdate2 = strftime("%b %02e", localtime($logtime));
my $repdate = strftime("%a, %e %b %Y", localtime($logtime));

my %traffic;
my @areas;
my $area;

my $total;
my $flag;

my $f_msg, $f_area, $f_traf;
my $f_r,      $f_i,      $f_d,      $f_c,      $f_ro;
my $f_traf_r, $f_traf_i, $f_traf_d, $f_traf_c, $f_traf_ro;



format HEADER =
^||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||||
$f_msg
.


format HEADER_summary =
               
                            Area name              Messages 
               
.

format LINE_summary =
                ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<  ^>>>>>>> 
                 $f_area,                            $f_traf
.

format FOOTER_summary =
               
                              TOTAL                ^>>>>>>> 
                                                     $f_traf
               
.


format HEADER_killed =
    
                 Area name                         Messages          
                                          r    i    d    c   r/o 
    
.

format LINE_killed =
     ^<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<  ^>>  ^>>  ^>>  ^>>  ^>> 
      $f_area, $f_traf_r, $f_traf_i, $f_traf_d, $f_traf_c, $f_traf_ro
.

format FOOTER_killed =
    
                   TOTAL                 ^>>  ^>>  ^>>  ^>>  ^>> 
    $f_traf_r, $f_traf_i, $f_traf_d, $f_traf_c, $f_traf_ro
    
    r   == routed
    i   == insecure
    d   == dupes
    c   == circular path
    r/o == from read only links
.



  foreach $log ("$logdir/log-in.1.gz","$logdir/log-in.1","$logdir/log-in")
  {

    # Check if logfile is readable
    #
    if (!(-r $log))
      {
	next;
      }

    # Try to open logfile
    #
    if ($log =~ /\.gz$/)
      {
	open(LOG, "gunzip -c $log |") || next;
      }
    else
      {
	open(LOG, "< $log") || next;
      }

    # Read logfile
    #
    while (<LOG>)
      {
	if (!($_ =~ ("^" . $logdate1)) && !($_ =~ ("^" . $logdate2)))
	  {
	    next;
	  }

	if ($_ =~ m( ftntoss area ([^ ]+) *: msgs in: ([0-9]+) *out: ([0-9]+) *killed: ([0-9]+)/([0-9]+)/([0-9]+)/([0-9]+)/([0-9]+)))
	  {
	    $area = uc($1);
	    $traffic{$area}{'in'}       += $2;
	    $traffic{$area}{'out'}      += $3;
	    $traffic{$area}{'routed'}   += $4;
	    $traffic{$area}{'insecure'} += $5;
	    $traffic{$area}{'dupes'}    += $6;
	    $traffic{$area}{'circular'} += $7;
	    $traffic{$area}{'ro'}       += $8;
	  }
	elsif ($_ =~ m( ftntoss area ([^ ]+) *: msgs in: ([0-9]+) *out: ([0-9]+) *killed: ([0-9]+)/([0-9]+)/([0-9]+)/([0-9]+)))
	  {
	    $area = uc($1);
	    $traffic{$area}{'in'}       += $2;
	    $traffic{$area}{'out'}      += $3;
	    $traffic{$area}{'routed'}   += $4;
	    $traffic{$area}{'insecure'} += $5;
	    $traffic{$area}{'dupes'}    += $6;
	    $traffic{$area}{'circular'} += $7;
	  }
	elsif ($_ =~ / ftntoss area ([^ ]+) *: msgs in: ([0-9]+)/)
	  {
	    $area = uc($1);
	    $traffic{$area}{'in'}      += $2;
	    $traffic{$area}{'out'}     += $2;
	  }
      }
    
    close(LOG);
  }

@areas = sort(keys(%traffic));

open(STDOUT, "| inews -h -O -S -R") || die("can't pipe report to inews");
#open(STDOUT, "| cat") || die("can't pipe report to cat");

# Print RFC headers
#
print "From: ", $msgfrom, "\n";
print "Newsgroups: ", $msgnewsgroup, "\n";
print "Subject: Traffic statistics for ", $repdate, "\n";
print "\n";

# Print date info
#
$~ = HEADER;
$f_msg = "Traffic statistics at node " . $msgnode . " for yesterday (" . $repdate . ")";
write;
print "\n";

# Print total message statistics #############################################
#
$~ = HEADER_summary;
write;

$~ = LINE_summary;

$total = 0;
foreach $i (@areas)
  {
    $f_area = $i;
    $f_traf = $traffic{$i}{'in'};
    write;
    $total += $traffic{$i}{'in'};
  }

$~ = FOOTER_summary;
$f_traf = $total;
write;


# Print detailed message statistics ##########################################
#
$flag = 0;
$total = 0;
$f_traf_r = 0;
$f_traf_i = 0;
$f_traf_d = 0;
$f_traf_c = 0;
$f_traf_ro = 0;
foreach $i (@areas)
  {
    if ($traffic{$i}{'routed'}   != 0 ||
	$traffic{$i}{'insecure'} != 0 ||
	$traffic{$i}{'dupes'}    != 0 ||
	$traffic{$i}{'circular'} != 0 ||
	$traffic{$i}{'ro'}       != 0)
      {
	if ($flag == 0)
	  {
	    print "\n";
	    $~ = HEADER_killed;
	    write;
	    $~ = LINE_killed;
	    $flag = 1;
	  }
	$f_area = $i;
	$f_r  = $traffic{$i}{'routed'};   $f_traf_r  += $traffic{$i}{'routed'};
	$f_i  = $traffic{$i}{'insecure'}; $f_traf_i  += $traffic{$i}{'insecure'};
	$f_d  = $traffic{$i}{'dupes'};    $f_traf_d  += $traffic{$i}{'dupes'};
	$f_c  = $traffic{$i}{'circular'}; $f_traf_c  += $traffic{$i}{'circular'};
	$f_ro = $traffic{$i}{'ro'};       $f_traf_ro += $traffic{$i}{'ro'};
	write;
      }
  }

if ($flag == 1)
  {
    $~ = FOOTER_killed;
    $f_r  = $f_traf_r;
    $f_i  = $f_traf_i;
    $f_d  = $f_traf_d;
    $f_c  = $f_traf_c;
    $f_ro = $f_traf_ro;
    write;
  }

##############################################################################

close(STDOUT);

exit(0);
